@@ -49,11 +49,49 @@ from rasa.shared.nlu.constants import (
     INTENT_NAME_KEY,
     ENTITY_ATTRIBUTE_ROLE,
     ENTITY_ATTRIBUTE_GROUP,
+    PREDICTED_CONFIDENCE_KEY,
+    INTENT_RANKING_KEY,
+    ENTITY_ATTRIBUTE_TEXT,
+    ENTITY_ATTRIBUTE_START,
+    ENTITY_ATTRIBUTE_CONFIDENCE,
+    ENTITY_ATTRIBUTE_END,
 )
 
 if TYPE_CHECKING:
+    from typing_extensions import TypedDict
+
     from rasa.shared.core.trackers import DialogueStateTracker
 
+    EntityPrediction = TypedDict(
+        "EntityPrediction",
+        {
+            ENTITY_ATTRIBUTE_TEXT: Text,
+            ENTITY_ATTRIBUTE_START: Optional[float],
+            ENTITY_ATTRIBUTE_END: Optional[float],
+            ENTITY_ATTRIBUTE_VALUE: Text,
+            ENTITY_ATTRIBUTE_CONFIDENCE: float,
+            ENTITY_ATTRIBUTE_TYPE: Text,
+            ENTITY_ATTRIBUTE_GROUP: Optional[Text],
+            ENTITY_ATTRIBUTE_ROLE: Optional[Text],
+            "additional_info": Any,
+        },
+        total=False,
+    )
+
+    IntentPrediction = TypedDict(
+        "IntentPrediction", {INTENT_NAME_KEY: Text, PREDICTED_CONFIDENCE_KEY: float,},
+    )
+    NLUPredictionData = TypedDict(
+        "NLUPredictionData",
+        {
+            INTENT: IntentPrediction,
+            INTENT_RANKING_KEY: List[IntentPrediction],
+            ENTITIES: List[EntityPrediction],
+            "message_id": Optional[Text],
+            "metadata": Dict,
+        },
+        total=False,
+    )
 logger = logging.getLogger(__name__)
 
 
@@ -369,7 +407,7 @@ class UserUttered(Event):
         text: Optional[Text] = None,
         intent: Optional[Dict] = None,
         entities: Optional[List[Dict]] = None,
-        parse_data: Optional[Dict[Text, Any]] = None,
+        parse_data: Optional["NLUPredictionData"] = None,
         timestamp: Optional[float] = None,
         input_channel: Optional[Text] = None,
         message_id: Optional[Text] = None,
@@ -410,7 +448,7 @@ class UserUttered(Event):
             # happens during training
             self.use_text_for_featurization = False
 
-        self.parse_data = {
+        self.parse_data: "NLUPredictionData" = {
             INTENT: self.intent,
             # Copy entities so that changes to `self.entities` don't affect
             # `self.parse_data` and hence don't get persisted
@@ -426,7 +464,7 @@ class UserUttered(Event):
     @staticmethod
     def _from_parse_data(
         text: Text,
-        parse_data: Dict[Text, Any],
+        parse_data: "NLUPredictionData",
         timestamp: Optional[float] = None,
         input_channel: Optional[Text] = None,
         message_id: Optional[Text] = None,
@@ -1088,7 +1126,8 @@ class ReminderScheduled(Event):
     def __str__(self) -> Text:
         """Returns text representation of event."""
         return (
-            f"ReminderScheduled(intent: {self.intent}, trigger_date: {self.trigger_date_time}, "
+            f"ReminderScheduled(intent: {self.intent}, "
+            f"trigger_date: {self.trigger_date_time}, "
             f"entities: {self.entities}, name: {self.name})"
         )
 
@@ -1473,6 +1512,14 @@ class ActionExecuted(Event):
 
         super().__init__(timestamp, metadata)
 
+    def __members__(self) -> Tuple[Optional[Text], Optional[Text], Text]:
+        meta_no_nones = {k: v for k, v in self.metadata.items() if v is not None}
+        return (
+            self.action_name,
+            self.action_text,
+            jsonpickle.encode(meta_no_nones),
+        )
+
     def __repr__(self) -> Text:
         """Returns event as string for debugging."""
         return "ActionExecuted(action: {}, policy: {}, confidence: {})".format(
@@ -1485,18 +1532,14 @@ class ActionExecuted(Event):
 
     def __hash__(self) -> int:
         """Returns unique hash for event."""
-        return hash(self.action_name or self.action_text)
+        return hash(self.__members__())
 
     def __eq__(self, other: Any) -> bool:
-        """Checks if object is equal to another."""
+        """Compares object with other object."""
         if not isinstance(other, ActionExecuted):
             return NotImplemented
 
-        equal = self.action_name == other.action_name
-        if hasattr(self, "action_text") and hasattr(other, "action_text"):
-            equal = equal and self.action_text == other.action_text
-
-        return equal
+        return self.__members__() == other.__members__()
 
     def as_story_string(self) -> Text:
         """Returns event in Markdown format."""
