@@ -35,6 +35,8 @@ ENTITY_ATTRIBUTE_ROLE = "role"
 ENTITY_ATTRIBUTE_VALUE = "value"
 ENTITY_ATTRIBUTE_START = "start"
 ENTITY_ATTRIBUTE_END = "end"
+ENTITY_ATTRIBUTE_TEXT = "text"
+ENTITY_ATTRIBUTE_CONFIDENCE = "confidence"
 NO_ENTITY_TAG = "O"
 SPLIT_ENTITIES_BY_COMMA = "split_entities_by_comma"
 SPLIT_ENTITIES_BY_COMMA_DEFAULT_VALUE = True
