@@ -118,7 +118,7 @@ class Validator:
             event.intent["name"]
             for story in self.story_graph.story_steps
             for event in story.events
-            if type(event) == UserUttered and event.intent_name is not None
+            if type(event) == UserUttered
         }
 
         for story_intent in stories_intents:
@@ -216,7 +216,8 @@ class Validator:
 
                 if event.action_name not in self.domain.action_names_or_texts:
                     rasa.shared.utils.io.raise_warning(
-                        f"The action '{event.action_name}' is used in the '{story.block_name}' block, but it "
+                        f"The action '{event.action_name}' is used in the "
+                        f"'{story.block_name}' block, but it "
                         f"is not listed in the domain file. You should add it to your "
                         f"domain file!",
                         docs=DOCS_URL_DOMAINS,
@@ -290,7 +291,8 @@ class Validator:
                     continue
                 else:
                     rasa.shared.utils.io.raise_warning(
-                        f"The form slot '{slot}' in form '{form}' is not present in the domain slots."
+                        f"The form slot '{slot}' in form '{form}' "
+                        f"is not present in the domain slots."
                         f"Please add the correct slot or check for typos.",
                         docs=DOCS_URL_DOMAINS,
                     )
@@ -309,7 +311,8 @@ class Validator:
         for intent_key, intent_dict in self.domain.intent_properties.items():
             if "triggers" in intent_dict:
                 rasa.shared.utils.io.raise_warning(
-                    f"The intent {intent_key} in the domain file is using the MappingPolicy format "
+                    f"The intent {intent_key} in the domain file "
+                    f"is using the MappingPolicy format "
                     f"which has now been deprecated. "
                     f"Please migrate to RulePolicy."
                 )
